function [FirstDeriv, SecondDeriv] = derivatives(y,a,THETA,h,TS)

% Compute the first and second derivative of GARCh variances with respect
% to the weigths of the portfolio (formulae 3 and 4 in text). 

% INPUT:    y = (T,N)-matrix of individual asset returns
%           a = (N-1,1)-vector of the first N-1 asset weigths
%           THETA = (3,1)-vector of GARCH parameter estimates
%           h = (T,1)-vector of estimated portfolio variances
%           TS = indicator (set equal 1 to produce the whole univariate
%           time series of first and second derivatives; if set equal to 
%           0 the output is only the first and second derivative at time T)

[T,N] = size(y);
Y = y(:,1:N-1)*a + y(:,N)*(1-a'*ones(N-1,1)); % (T,1)-vector of portfolio returns

% ****************************************************************
% Initialise the variables
% ****************************************************************
z = [ones(1,T); (Y.^2)'; h']; z = [zeros(3,1), z(:,1:end-1)];
LTT = 0; LTa = 0; dLTTda = 0; dLTada = 0;
dhTda = zeros(N-1,1); dhdTTda = zeros(3,N-1); dhdTdTTda = zeros(9,N-1); dhdTdaTda = zeros(3*(N-1),N-1); dzdaTda = zeros(3*(N-1),N-1); dhdaTda = zeros(N-1,N-1);
dhda = zeros(1,N-1); dzda = zeros(3,N-1);
dhdT = zeros(3,1); dhdTdT = zeros(3,3); dhdTda = zeros(3,N-1); dhTdada = zeros(N-1,N-1);
dhdTdTda = zeros(9,N-1); dhdada = zeros(N-1,N-1); dhdTdada = zeros(3*(N-1),N-1); dzdada = zeros(3*(N-1),N-1);

% We need to compute dTHETAda and dTHETAdada first
for t = 2:T
    % Apply theorem 1 to compute dTHETAda ...
    H = h(t)^(-1) - Y(t)^2*h(t)^(-2); % Formula (8)
    dzdT = [zeros(3,2), dhdT]; % Formula (10)
    dhdT = z(:,t) + dzdT * THETA; % Formula (9)
    Htilde = -h(t)^(-2) + 2*Y(t)^2*h(t)^(-3); % Formula (13)
    dHdT = dhdT' * Htilde; % Formula (12)
    G = [zeros(6,3); dhdTdT]; % Formula (15)
    dhdTdT = dzdT' + dzdT + kron(THETA', eye(3))*G; % Formula (14)
    LTT = LTT - 0.5*(dhdT*dHdT + H*dhdTdT); % Formula (11)
    
    dY2lagda = 2*Y(t-1)*(y(t-1,1:N-1)' - ones(N-1,1)*y(t-1,N)); 
    dY2da = 2*Y(t)*(y(t,1:N-1)' - ones(N-1,1)*y(t,N)); 
    dzda = [zeros(1,N-1); dY2lagda'; dhda]; % Formula (19)
    F = [zeros(6,N-1); dhdTda]; % Formula (21)
    dhdTda = dzda + kron(THETA',eye(3))*F; % Formula (20)
    dhda = THETA'*dzda; % Formula (18)
    dHda = dhda*Htilde - h(t)^(-2)*dY2da'; % Formula (17)
    LTa = LTa - 0.5*(dhdT*dHda +H*dhdTda); % Formula (16)
end

dTda = -LTa'*inv(LTT); % Formula (4)

% Compute the first derivative of variance with respect to weigths
%for t = 2:T
%    dY2lagda = 2*Y(t-1) * (y(t-1,1:N-1)' - ones(N-1,1)*y(t-1,N));
%    dzTda = [zeros(N-1,1), dY2lagda, dhTda]; % Formula (2)
%    dhTda = dzTda*THETA + dTda*z(:,t); % Formula (1)
%end

% Re-initialise some variables
dhTda = 0*dhTda; dhdTdT = 0*dhdTdT; dhda = 0*dhda; dhdT = 0*dhdT; dhdTda = 0*dhdTda;

for t = 2:T
    % ... and to compute dTHETAdada
    H = h(t)^(-1) - Y(t)^2*h(t)^(-2); % Formula (8)
    Htilde = -h(t)^(-2) + 2*Y(t)^2*h(t)^(-3); % Formula (13)
    dzdT = [zeros(3,2), dhdT]; % Formula (10)
    dhdT = z(:,t) + dzdT * THETA; % Formula (9)
    dHdT = dhdT' * Htilde; % Formula (12)
    dY2lagda = 2*Y(t-1)*(y(t-1,1:N-1)' - ones(N-1,1)*y(t-1,N)); 
    dzTda = [zeros(N-1,1), dY2lagda, dhTda]; % Formula (2)
    dhTda = dzTda*THETA + dTda*z(:,t); % Formula (1)
    dzdTTda = [zeros(6,N-1); dhdTTda]; % Formula (28)
    dhdTTda = dzTda' + dzdT*dTda' + kron(THETA',eye(3))*dzdTTda; % Formula (27)
    dY2da = 2*Y(t)*(y(t,1:N-1)' - ones(N-1,1)*y(t,N)); 
    dHTda = dhTda'*Htilde - h(t)^(-2)*dY2da'; % Formula (29)
    dHtildeTda = (2*h(t)^(-3) - 6*Y(t)^2*h(t)^(-4)) * dhTda' + 2*h(t)^(-3)*dY2da'; % Formula (31)
    dHdTTda = dhdTTda*Htilde + dhdT*dHtildeTda; % Formula (30)
    dzdTpTda = commutation(3,3)*dzdTTda; % Formula (33)
    I3 = eye(3);
    G = [zeros(6,3); dhdTdT]; % Formula (15)
    dhdTdT = dzdT' + dzdT + kron(THETA', eye(3))*G; % Formula (14)
    L1 = kron(eye(3),kron(commutation(1,3),eye(3))) * kron(I3(:),dTda'); % Formula (35)
    L2 = [zeros(18,N-1); dhdTdTTda]; % Formula (36)
    dhdTdTTda = dzdTTda + dzdTpTda + kron(eye(3),G') * L1 + kron(kron(eye(3),THETA'),eye(3)) * L2; % Formula (32)
    dLTTda = dLTTda - 0.5*(kron(eye(3),dhdT)*dHdTTda + kron(dHdT',eye(3))*dhdTTda + dhdTdT(:)*dHTda + H*dhdTdTTda); % Formula (26)
    
    F = [zeros(6,N-1); dhdTda]; % Formula (21)
    dzda = [zeros(1,N-1); dY2lagda'; dhda]; % Formula (19)
    dhda = THETA'*dzda; % Formula (18)
    dhdTda = dzda + kron(THETA',eye(3))*F; % Formula (20)
    J1 = kron(commutation(3,3),eye(3)) * kron(dTda',I3(:)); % Formula (42)
    J2 = [zeros(6*(N-1),N-1); dhdTdaTda]; % Formula (43)
    dY2lagdada = 2*(y(t-1,1:N-1)' - ones(N-1,1)*y(t-1,N))*(y(t-1,1:N-1)' - ones(N-1,1)*y(t-1,N))';
    dzdaTda = [zeros(N-1,N-1); dY2lagdada; dhdaTda]; % Formula (41)
    dhdTdaTda = dzdaTda + kron(eye(3),F') * J1 + kron(kron(THETA',eye(3)),eye(N-1)) * J2; % Formula (41)
    dhdaTda = dzda'*dTda' + kron(THETA',eye(N-1))*dzdaTda; % Formula (39)
    dY2dada = 2*(y(t,1:N-1)' - ones(N-1,1)*y(t,N))*(y(t,1:N-1)' - ones(N-1,1)*y(t,N))';
    dHdaTda = Htilde*dhdaTda + dhda'*dHtildeTda - h(t)^(-2)*dY2dada + 2*h(t)^(-3)*dY2da*dhTda'; % Formula (38)
    dhdTpda = dhdTda';
    dHda = dhda*Htilde - h(t)^(-2)*dY2da'; % Formula (17)
    dLTada = dLTada - 0.5 * (kron(eye(3),dHda')*dhdTTda + kron(dhdT,eye(N-1))*dHdaTda + dhdTpda(:)*dHTda + H*dhdTdaTda); % Formula (37)
end

dTdada = - kron(inv(LTT),dTda)*dLTTda - kron(inv(LTT),eye(N-1))*dLTada; % Formula (25)

% Compute the second derivative of variance with respect to weigths
dhTda = 0*dhTda;
if TS, dhTda = zeros(T,1); dhTdada = zeros(T,1); end
for t = 2:T
    dY2lagda = 2*Y(t-1)*(y(t-1,1:N-1)' - ones(N-1,1)*y(t-1,N)); 
    dY2lagdada = 2*(y(t-1,1:N-1)' - ones(N-1,1)*y(t-1,N))*(y(t-1,1:N-1)' - ones(N-1,1)*y(t-1,N))'; % Formula (24)
    if TS
        dzTdada = [zeros(N-1,N-1); dY2lagdada; dhTdada(t-1,1)]; % Formula (23)
        dzTda = [zeros(N-1,1), dY2lagda, dhTda(t-1,1)]; % Formula (2)
        dhTda(t,1) = dzTda*THETA + dTda*z(:,t); % Formula (1)
        dhTdada(t,1) = dzTda*dTda' + dTda*dzTda' + kron(THETA',eye(N-1))*dzTdada + kron(z(:,t)',eye(N-1))*dTdada; % Formula (22)
    else
        dzTdada = [zeros(N-1,N-1); dY2lagdada; dhTdada]; % Formula (23)
        dzTda = [zeros(N-1,1), dY2lagda, dhTda]; % Formula (2)
        dhTda = dzTda*THETA + dTda*z(:,t); % Formula (1)
        dhTdada = dzTda*dTda' + dTda*dzTda' + kron(THETA',eye(N-1))*dzTdada + kron(z(:,t)',eye(N-1))*dTdada; % Formula (22)
    end
end
FirstDeriv = dhTda;
SecondDeriv = dhTdada;
