function [h, firstDeriv, secondDeriv] = GARCHsensitivity(a,y)

% Estimate a GARCH(1,1) model.
% The input is a (T,N) matrix of individual stock returns.
% The output contains the likelihood value, the parameter estimates, the exitflag, and the 
% time series of estimated variances.

[T,N] = size(y);
Y = y(:,1:N-1)*a + y(:,N)*(1-a'*ones(N-1,1)); % (T,1)-vector of portfolio returns

options = optimset('LargeScale', 'off', 'Display', 'off', 'MaxFunEvals', 500, 'MaxIter', 200 , 'TolFun', 1e-10, 'TolX', 1e-10);

% Initialise variance.
LAMBDA    = 0.94;
expsmooth = LAMBDA .^ [0: 1 : T-1] * Y .^ 2;
sigma2    = LAMBDA ^ T * (Y'*Y/T) + (1-LAMBDA) * expsmooth;
%sigma2=1;

% Find the best starting values.
nInitialVectors      = [1000, 3]; % Number of initial vector fed in the uniform random number generator SAV and GARCH models.
nInitialCond         = 2; % Select the number of initial conditions for the optimisation.
ParamEstimates       = zeros(nInitialCond, 3);
initialTargetVectors = unifrnd(0, 1, nInitialVectors); 
%initialTargetVectors = [unifrnd(0,3,1000,1), unifrnd(0,.4,1000,1), unifrnd(.4,3,1000,1)];
fvalTemp             = zeros(nInitialVectors(1), 1);
for i = 1:nInitialVectors(1)
   fvalTemp(i,1) = GARCHlik(initialTargetVectors(i,:), Y, sigma2);
end
Results          = [fvalTemp, initialTargetVectors];
SortedResults    = sortrows(Results,1);
BestInitialCond  = SortedResults(1:nInitialCond,2:4);

% Run the optimisation algorithm for each of the initial conditions.
for i = 1:size(BestInitialCond,1)
   [ParamEstimates(i,:), fval(i), exitflag(i)] = fminsearch('GARCHlik', BestInitialCond(i,:), options, Y, sigma2);
   for kk = 1:10
      [ParamEstimates(i,:), fval(i), exitflag(i)] = fminsearch('GARCHlik', ParamEstimates(i,:), options, Y, sigma2);            
      if exitflag(i) == 1
         break
      end
   end
end

% Pick the estimate with the best likelihood.
SortedFval  = sortrows([fval', ParamEstimates, exitflag', BestInitialCond], 1);
f.fvalsmplx = SortedFval(1, 1);
f.theta     = SortedFval(1, 2:4)';
f.exitflag  = SortedFval(1, 5);

% Compute the time series of estimated variances.
f.variance = varianceLoop(f.theta, Y, sigma2, 3); 

[firstDeriv, secondDeriv] = Derivatives(y, a, f.theta, f.variance, 0);
h = f.variance(end);